/*
	orxonox - the future of 3D-vertical-scrollers

	Copyright (C) 2004 orx

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2, or (at your option)
	any later version.

### File Specific:
	main-programmer: hdavid, amaechler
*/

#include "volfog_effect.h"

#include "util/loading/load_param.h"
#include "util/loading/factory.h"

#include "glincl.h"
//#include "shell_command.h"

#define GLX_GLXEXT_PROTOTYPES
#include <GL/glx.h>

//#include <GL/glext.h> //OpenGL Extensions
//#include <GL/glxext.h> // GLX Extensions 

#ifndef GL_EXT_fog_coord
#define GL_EXT_fog_coord 1

typedef void (APIENTRY * PFNGLFOGCOORDFEXTPROC) (GLfloat coord);
typedef void (APIENTRY * PFNGLFOGCOORDFVEXTPROC) (const GLfloat *coord);
typedef void (APIENTRY * PFNGLFOGCOORDDEXTPROC) (GLdouble coord);
typedef void (APIENTRY * PFNGLFOGCOORDDVEXTPROC) (const GLdouble *coord);
typedef void (APIENTRY * PFNGLFOGCOORDPOINTEREXTPROC) (GLenum type, GLsizei stride, const GLvoid *pointer);
#endif

PFNGLFOGCOORDFEXTPROC glFogCoordfEXT = 0;
PFNGLFOGCOORDFVEXTPROC glFogCoordfvEXT = 0;
PFNGLFOGCOORDDEXTPROC glFogCoorddEXT = 0;
PFNGLFOGCOORDDVEXTPROC glFogCoorddvEXT = 0;
PFNGLFOGCOORDPOINTEREXTPROC glFogCoordPointerEXT = 0;


using namespace std;

CREATE_FACTORY(VolFogEffect, CL_VOLFOG_EFFECT);

VolFogEffect::VolFogEffect(const TiXmlElement* root)
{
	this->setClassID(CL_VOLFOG_EFFECT, "VolFogEffect");

	if (root != NULL)
		this->loadParams(root);

	// Initialize Effect
	this->init();

	// Activate Effect
	this->activate();
}


VolFogEffect::~VolFogEffect()
{
	this->deactivate();
}

void VolFogEffect::loadParams(const TiXmlElement* root)
{
	WeatherEffect::loadParams(root);
}

bool VolFogEffect::init()
{
	PRINTF(0)("Initalize VolFogEffect\n");

	// set fog mode
	GLenum fogMode  = GL_EXP2;

	// set fog density
	float  fogDensity  = 0.03f;

	// set fog near & far distance
	float fogStart = 0.05f;
	float fogEnd   = 100.0f;

	// Set fog color
	float fogColor[4] = {0.0, 1.0, 0.0, 1.0};
	
	glClearColor(fogColor[0],fogColor[1],fogColor[2],0.0f);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);

	glFogCoordfEXT       = (PFNGLFOGCOORDFEXTPROC)glXGetProcAddressARB((const GLubyte*)"glFogCoordfEXT");
	glFogCoordfvEXT      = (PFNGLFOGCOORDFVEXTPROC)glXGetProcAddressARB((const GLubyte*)"glFogCoordfvEXT");
	glFogCoorddEXT       = (PFNGLFOGCOORDDEXTPROC)glXGetProcAddressARB((const GLubyte*)"glFogCoorddEXT");
	glFogCoorddvEXT      = (PFNGLFOGCOORDDVEXTPROC)glXGetProcAddressARB((const GLubyte*)"glFogCoorddvEXT");
	glFogCoordPointerEXT = (PFNGLFOGCOORDPOINTEREXTPROC)glXGetProcAddressARB((const GLubyte*)"glFogCoordPointerEXT");

	// set the fog attributes
	glFogf (GL_FOG_START,  fogStart);
	glFogf (GL_FOG_END,    fogEnd);
	glFogfv(GL_FOG_COLOR,  fogColor);      
	glFogi (GL_FOG_MODE,   fogMode);
	glFogf (GL_FOG_DENSITY,fogDensity);
	glFogi (GL_FOG_COORDINATE_SOURCE_EXT,GL_FOG_COORDINATE_EXT);

	// enable the fog
	glEnable(GL_FOG);

	if (glewInit() == GLEW_OK)
		PRINTF(0)("glewInit OK\n");
	else
		PRINTF(0)("glewInit failed\n");

	if (glewGetExtension("GL_EXT_fog_coord"))
	{
		PRINTF(0)("GL_EXT_fog_coord extension found\n");
		return true;
	}
	else
	{
		PRINTF(0)("GL_EXT_fog_coord extension NOT found\n");
		return false;
	}
}


bool VolFogEffect::activate()
{
	PRINTF(0)("Activating VolFogEffect\n");
}

bool VolFogEffect::deactivate()
{
	PRINTF(0)("Deactivating VolFogEffect\n");
}


/**
* draws the effect, if needed
*/
void VolFogEffect::draw() const
{
	
	/* clear all pixels in colour buffer */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	/* Enable blending */
	//glEnable(GL_BLEND);
	//glBlendFunc(GL_SRC_ALPHA, GL_DST_ALPHA);

	/* draw the panels */
	glBegin(GL_QUADS); // Floor
		glFogCoordfEXT(0.0f);	glVertex3f(0.0f,0.0f,0.0f);
		glFogCoordfEXT(0.0f);	glVertex3f(100.0f,0.0f,0.0f);
		glFogCoordfEXT(0.0f);	glVertex3f(100.0f,0.0f, 100.0f);
		glFogCoordfEXT(0.0f);	glVertex3f(0.0f,0.0f, 100.0f);
	glEnd();

	glBegin(GL_QUADS); // Roof
		glFogCoordfEXT(1.0f);	glVertex3f(0.0f, 100.0f,0.0f);
		glFogCoordfEXT(1.0f);	glVertex3f( 100.0f, 100.0f,0.0f);
		glFogCoordfEXT(1.0f);	glVertex3f( 100.0f, 100.0f, 100.0f);
		glFogCoordfEXT(1.0f);	glVertex3f(0.0f, 100.0f, 100.0f);
	glEnd();

	glBegin(GL_QUADS); // Back Wall
		glFogCoordfEXT(0.0f);	glVertex3f(0.0f,0.0f,0.0f);
		glFogCoordfEXT(0.0f);	glVertex3f( 100.0f,0.0f,0.0f);
		glFogCoordfEXT(1.0f);	glVertex3f( 100.0f, 100.0f,0.0f);
		glFogCoordfEXT(1.0f);	glVertex3f(0.0f, 100.0f,0.0f);
	glEnd();

	glBegin(GL_QUADS); // Front Wall
		glFogCoordfEXT(0.0f);	glVertex3f(0.0f,0.0f,100.0f);
		glFogCoordfEXT(0.0f);	glVertex3f( 100.0f,0.0f,100.0f);
		glFogCoordfEXT(1.0f);	glVertex3f( 100.0f, 100.0f,100.0f);
		glFogCoordfEXT(1.0f);	glVertex3f(0.0f, 100.0f,100.0f);
	glEnd();

	glBegin(GL_QUADS); // Right Wall
		glFogCoordfEXT(0.0f);	glVertex3f( 100.0f,0.0f, 100.0f);
		glFogCoordfEXT(1.0f);	glVertex3f( 100.0f, 100.0f, 100.0f);
		glFogCoordfEXT(1.0f);	glVertex3f( 100.0f, 100.0f,0.0f);
		glFogCoordfEXT(0.0f);	glVertex3f( 100.0f,0.0f,0.0f);
	glEnd();

	glBegin(GL_QUADS); // Left Wall
		glFogCoordfEXT(0.0f);	glVertex3f(0.0f,0.0f, 100.0f);
		glFogCoordfEXT(1.0f);	glVertex3f(0.0f, 100.0f, 100.0f);
		glFogCoordfEXT(1.0f);	glVertex3f(0.0f, 100.0f,0.0f);
		glFogCoordfEXT(0.0f);	glVertex3f(0.0f,0.0f,0.0f);
	glEnd();
	
	/* process all buffered OpenGL commands */
	glFlush();

}


/**
* ticks the effect if there is any time dependancy
*/
void VolFogEffect::tick(float dt)	
{
}
